@@ -41,14 +41,19 @@ impl ::core::ops::DerefMut for ProgramOutput {
 
 pub fn interpret<R: Rng>(
     rng: &mut R,
-    Program { tree: Tree { arena, top }}: &Program,
+    Program {
+        tree: Tree { arena, top },
+    }: &Program,
 ) -> Result<ProgramOutput, InterpError> {
     let mut outputs = Arena::new();
-    let top = interpret_term(rng, &arena, &mut outputs, *top)?;
+    let top = interpret_term(rng, arena, &mut outputs, *top)?;
     let total = outputs[top].1.total();
     Ok(ProgramOutput {
         total,
-        tree: Tree { arena: outputs, top },
+        tree: Tree {
+            arena: outputs,
+            top,
+        },
     })
 }
 
@@ -74,12 +79,12 @@ impl TermOutput {
     fn total(&self) -> i64 {
         *match self {
             TermOutput::Constant(total)
-                | TermOutput::DiceRoll(total, ..)
-                | TermOutput::KeepHigh(KeepHigh { total, .. })
-                | TermOutput::Add(total, ..)
-                | TermOutput::Subtract(total, ..)
-                | TermOutput::UnarySubtract(total, ..)
-                | TermOutput::UnaryAdd(total, ..) => total,
+            | TermOutput::DiceRoll(total, ..)
+            | TermOutput::KeepHigh(KeepHigh { total, .. })
+            | TermOutput::Add(total, ..)
+            | TermOutput::Subtract(total, ..)
+            | TermOutput::UnarySubtract(total, ..)
+            | TermOutput::UnaryAdd(total, ..) => total,
         }
     }
 }
@@ -99,7 +104,7 @@ fn interpret_term<R: Rng>(
                 let mut total: i64 = 0;
                 let mut parts = Vec::with_capacity(count as usize);
                 for _ in 0..count {
-                    let random = rng.gen_range(0, sides) + 1;
+                    let random = rng.gen_range(1..=sides);
                     total = total
                         .checked_add(random)
                         .ok_or(InterpError::OverflowPositive)?;
@@ -116,33 +121,44 @@ fn interpret_term<R: Rng>(
                         use ::core_extensions::SliceExt;
                         partials.sort_unstable_by(|a, b| b.cmp(a));
                         let total = partials.slice_lossy(0..(count as _), ()).iter().sum();
-                        Ok(term_outputs.alloc((term, TermOutput::KeepHigh(KeepHigh {
-                            total,
-                            // Note that the saved `keep_count` isn't guaranteed
-                            // to match the actual number of partial sums.
-                            // 4d6k5 will only keep 4 dice, but `keep_count` will be 5.
-                            keep_count: count,
-                            roll,
-                        }))))
+                        Ok(term_outputs.alloc((
+                            term,
+                            TermOutput::KeepHigh(KeepHigh {
+                                total,
+                                // Note that the saved `keep_count` isn't guaranteed
+                                // to match the actual number of partial sums.
+                                // 4d6k5 will only keep 4 dice, but `keep_count` will be 5.
+                                keep_count: count,
+                                roll,
+                            }),
+                        )))
+                    }
+                    None => {
+                        if count >= 1 {
+                            let total = *total;
+                            Ok(term_outputs.alloc((
+                                term,
+                                TermOutput::KeepHigh(KeepHigh {
+                                    total,
+                                    // Note that we preserve the behavior of the above branch,
+                                    // where `keep_count` is the same as requested,
+                                    // not what is actually received, here.
+                                    // d6k3 will only keep 1 die, but `keep_count` will be 3.
+                                    keep_count: count,
+                                    roll,
+                                }),
+                            )))
+                        } else {
+                            Ok(term_outputs.alloc((
+                                term,
+                                TermOutput::KeepHigh(KeepHigh {
+                                    total: 0,
+                                    keep_count: count,
+                                    roll,
+                                }),
+                            )))
+                        }
                     }
-                    None => if count >= 1 {
-                        let total = *total;
-                        Ok(term_outputs.alloc((term, TermOutput::KeepHigh(KeepHigh {
-                            total,
-                            // Note that we preserve the behavior of the above branch,
-                            // where `keep_count` is the same as requested,
-                            // not what is actually received, here.
-                            // d6k3 will only keep 1 die, but `keep_count` will be 3.
-                            keep_count: count,
-                            roll,
-                        }))))
-                    } else {
-                        Ok(term_outputs.alloc((term, TermOutput::KeepHigh(KeepHigh {
-                            total: 0,
-                            keep_count: count,
-                            roll,
-                        }))))
-                    },
                 },
                 _ => unreachable!("nesting of dice operators is currently not permitted"),
             }
@@ -150,21 +166,21 @@ fn interpret_term<R: Rng>(
         Term::Add(left, right) => {
             let (total, left, right) = interpret_term(&mut *rng, terms, &mut *term_outputs, left)
                 .and_then(|left| {
-                    interpret_term(&mut *rng, terms, &mut *term_outputs, right).and_then(|right| {
-                        let (left_total, right_total) =
-                            (term_outputs[left].1.total(), term_outputs[right].1.total());
-                        left_total
-                            .checked_add(right_total)
-                            .ok_or_else(|| {
-                                if left_total > 0 || right_total > 0 {
-                                    InterpError::OverflowPositive
-                                } else {
-                                    InterpError::OverflowNegative
-                                }
-                            })
-                            .map(|total| (total, left, right))
-                    })
-                })?;
+                interpret_term(&mut *rng, terms, &mut *term_outputs, right).and_then(|right| {
+                    let (left_total, right_total) =
+                        (term_outputs[left].1.total(), term_outputs[right].1.total());
+                    left_total
+                        .checked_add(right_total)
+                        .ok_or_else(|| {
+                            if left_total > 0 || right_total > 0 {
+                                InterpError::OverflowPositive
+                            } else {
+                                InterpError::OverflowNegative
+                            }
+                        })
+                        .map(|total| (total, left, right))
+                })
+            })?;
             Ok(term_outputs.alloc((term, TermOutput::Add(total, left, right))))
         }
         Term::Subtract(left, right) => {
@@ -172,45 +188,57 @@ fn interpret_term<R: Rng>(
             let right = interpret_term(&mut *rng, terms, &mut *term_outputs, right)?;
             let (left_total, right_total) =
                 (term_outputs[left].1.total(), term_outputs[right].1.total());
-            Ok(term_outputs.alloc((term, TermOutput::Subtract(
-                left_total.checked_sub(right_total).ok_or_else(|| {
-                    if left_total > 0 || right_total < 0 {
-                        InterpError::OverflowPositive
-                    } else {
-                        InterpError::OverflowNegative
-                    }
-                })?,
-                left,
-                right,
-            ))))
+            Ok(term_outputs.alloc((
+                term,
+                TermOutput::Subtract(
+                    left_total.checked_sub(right_total).ok_or_else(|| {
+                        if left_total > 0 || right_total < 0 {
+                            InterpError::OverflowPositive
+                        } else {
+                            InterpError::OverflowNegative
+                        }
+                    })?,
+                    left,
+                    right,
+                ),
+            )))
         }
         Term::UnarySubtract(term_0) => {
             let term_0 = interpret_term(&mut *rng, terms, &mut *term_outputs, term_0)?;
             let term_total = term_outputs[term_0].1.total();
-            Ok(term_outputs.alloc((term, TermOutput::UnarySubtract(
-                term_total
-                    .checked_neg()
-                    .ok_or(InterpError::OverflowNegative)?,
-                term_0,
-            ))))
+            Ok(term_outputs.alloc((
+                term,
+                TermOutput::UnarySubtract(
+                    term_total
+                        .checked_neg()
+                        .ok_or(InterpError::OverflowNegative)?,
+                    term_0,
+                ),
+            )))
         }
         Term::UnaryAdd(term_0) => {
             let term_0 = interpret_term(&mut *rng, terms, &mut *term_outputs, term_0)?;
             let term_total = term_outputs[term_0].1.total();
             Ok(term_outputs.alloc((term, TermOutput::UnaryAdd(term_total, term_0))))
         }
+        ref term => unimplemented!("evaluating term {:?}", term),
     }
 }
 
 pub mod fmt {
-    use ::id_arena::{Id, Arena};
-    use super::ProgramOutput;
     use super::KeepHigh;
-    use super::TermOutput;
     use super::Out;
+    use super::ProgramOutput;
+    use super::TermOutput;
     use crate::parse::Term;
+    use ::id_arena::{Arena, Id};
 
-    fn fmt_default_impl(buf: &mut String, current: Id<Out>, arena: &Arena<Out>, terms: &Arena<Term>) {
+    fn fmt_default_impl(
+        buf: &mut String,
+        current: Id<Out>,
+        arena: &Arena<Out>,
+        terms: &Arena<Term>,
+    ) {
         let (term, out) = &arena[current];
         match out {
             TermOutput::Constant(n) => ::itoa::fmt(buf, *n).unwrap(),
@@ -237,7 +265,7 @@ pub mod fmt {
                                 buf.push_str(" + ");
                                 itoa::fmt(&mut *buf, *part).unwrap();
                             }
-                        },
+                        }
                         Some([]) => unreachable!("groups of zero dice"),
                         None => match count {
                             0 => unreachable!("group of zero dice"),
@@ -246,8 +274,8 @@ pub mod fmt {
                                 for _ in 1..count {
                                     buf.push_str(" + 1");
                                 }
-                            },
-                        }
+                            }
+                        },
                     }
                     buf.push(')');
                 } else {
@@ -255,8 +283,12 @@ pub mod fmt {
                     buf.push('d');
                     itoa::fmt(&mut *buf, sides).unwrap();
                 }
-            },
-            TermOutput::KeepHigh(KeepHigh { total: _, keep_count, roll }) => {
+            }
+            TermOutput::KeepHigh(KeepHigh {
+                total: _,
+                keep_count,
+                roll,
+            }) => {
                 // Same TODO as above. Restructure a bit so we don't need these unwraps.
                 let (term, _) = terms[*term].clone().unwrap_keep_high();
                 let (count, sides) = terms[term].clone().unwrap_dice_roll();
@@ -290,7 +322,7 @@ pub mod fmt {
                             if i <= *keep_count as usize {
                                 buf.push_str("**");
                             }
-                        },
+                        }
                         Some([]) => unreachable!("groups of zero dice"),
                         None => match count {
                             0 => unreachable!("group of zero dice"),
@@ -308,8 +340,8 @@ pub mod fmt {
                                 if i <= *keep_count as usize {
                                     buf.push_str("**");
                                 }
-                            },
-                        }
+                            }
+                        },
                     }
                     buf.push(')');
                 } else {
@@ -319,21 +351,21 @@ pub mod fmt {
                     buf.push('k');
                     itoa::fmt(&mut *buf, *keep_count).unwrap();
                 }
-            },
+            }
             TermOutput::Add(_total, left, right) => {
                 fmt_default_impl(&mut *buf, *left, arena, terms);
                 buf.push_str(" + ");
                 fmt_default_impl(&mut *buf, *right, arena, terms);
-            },
+            }
             TermOutput::Subtract(_total, left, right) => {
                 fmt_default_impl(&mut *buf, *left, arena, terms);
                 buf.push_str(" - ");
                 fmt_default_impl(&mut *buf, *right, arena, terms);
-            },
+            }
             TermOutput::UnarySubtract(_total, only) => {
                 buf.push('-');
                 fmt_default_impl(buf, *only, arena, terms);
-            },
+            }
             TermOutput::UnaryAdd(_total, only) => fmt_default_impl(buf, *only, arena, terms),
         }
     }
@@ -365,8 +397,12 @@ pub mod fmt {
                     buf.push('d');
                     itoa::fmt(&mut *buf, sides).unwrap();
                 }
-            },
-            TermOutput::KeepHigh(KeepHigh { total, keep_count, roll }) => {
+            }
+            TermOutput::KeepHigh(KeepHigh {
+                total,
+                keep_count,
+                roll,
+            }) => {
                 // Same TODO as above. Restructure a bit so we don't need these unwraps.
                 let (term, _) = terms[*term].clone().unwrap_keep_high();
                 let (count, sides) = terms[term].clone().unwrap_dice_roll();
@@ -393,21 +429,21 @@ pub mod fmt {
                     buf.push('k');
                     itoa::fmt(&mut *buf, *keep_count).unwrap();
                 }
-            },
+            }
             TermOutput::Add(_total, left, right) => {
                 fmt_short_impl(&mut *buf, *left, arena, terms);
                 buf.push_str(" + ");
                 fmt_short_impl(&mut *buf, *right, arena, terms);
-            },
+            }
             TermOutput::Subtract(_total, left, right) => {
                 fmt_short_impl(&mut *buf, *left, arena, terms);
                 buf.push_str(" - ");
                 fmt_short_impl(&mut *buf, *right, arena, terms);
-            },
+            }
             TermOutput::UnarySubtract(_total, only) => {
                 buf.push('-');
                 fmt_short_impl(buf, *only, arena, terms);
-            },
+            }
             TermOutput::UnaryAdd(_total, only) => fmt_short_impl(buf, *only, arena, terms),
         }
     }
